#########################################################################################
# Exonmap is a Package to provide fine grained annotation of Affymetrix exon microarrays.
# It makes use of a MySQL database containing the annotation
# 
# Please see the README.TXT file in this installation.
#
# Authors: M. Okoniewski, C.J. Miller
#########################################################################################

#####FUNCTIONS TO CREATE PRETTY PLOTS OF INDIVIDUAL GENES

### create a scale

.paint.scale <- function(tick.height,scale.colour,label.colour,tick.labels,cex=0.5) {
  ticks <- axTicks(1)
  segments(ticks,cex/2,ticks,tick.height,col=scale.colour,lty=1)
  segments(ticks,-cex/2,ticks,-tick.height,col=scale.colour,lty=1)

  if(tick.labels) {
    text(ticks,0,ticks,col=label.colour,cex=cex)
  }
}

### the plot area is a graph with hidden axes and a chromosome represented asa  horizontal line with a scale applied to it

.initialize.plot.area <- function(xlim,ylim=c(-1,1),xlab="",ylab="",draw.chromosome=T,chromosome.colour="black",chromosome.height=0.5,draw.scale=T,scale.colour="black",tick.height=1,tick.labels=T,label.colour="white",cex=cex) {
  plot(0,0,type="n",xaxt="n",yaxt="n",bty="n",xlim=xlim,ylim=ylim,xlab=xlab,ylab=ylab)
  if(draw.chromosome) { 
    rect(xleft=xlim[1],ybot=-chromosome.height/2,xright=xlim[2],ytop=chromosome.height/2,col=chromosome.colour,border=chromosome.colour,lty=1)
  }
  if(draw.scale) {
    .paint.scale(tick.height,scale.colour,label.colour,tick.labels,cex)
  }
}

.paint.probeset <- function(probes,offset=0.25,probeset.height=0.75,strand,border="black",fill="white",cex=0.3,draw.probes=F,lty=1) {
    r <- range(probes)

    rect(r[1],strand * offset,r[2],strand*(offset + probeset.height),border=border,col=fill,density=-1,lty=lty)
    if(draw.probes){
      points(probes,rep((strand * (offset + probeset.height/2)),length(probes)),pch=20,col=border,cex=cex)
    }
}

.gene.height <- function(transcripts=1,strand=1,offset=0.5,transcript.height=1,transcript.pad=0.1,label.height=0.5,pad.for.label=T,cex=0.5) {
     label.pad <- 0
     if(pad.for.label) { label.pad <- label.height + transcript.pad}

     yb <-  offset
     yt <- (offset + (transcript.pad + transcript.height) * transcripts + label.pad + 2 * transcript.pad)
     if(strand == -1 ) {
       tmp <- -yb
       yb  <- -yt
       yt  <- tmp
     }
     c(yb,yt)
}



### paint an individual probe

.paint.probe <- function(probe,offset=0.25,probe.height=0.75,strand,colour="grey",cex=0.3,lty=1) {
    rect(probe[1],strand*offset,probe[2],strand*2*offset,border="white",col=colour,density=-1)
    segments(probe[1]+(probe[2]-probe[1])/2,strand * offset,probe[1]+(probe[2]-probe[1])/2,strand*(offset + probe.height),col=colour,lty=lty)
}

### ... and a gene ...

.paint.gene <- function(xrange,transcripts=1,strand=1,offset=0.5,transcript.height=1,transcript.pad=0.1,border="black",fill="white",lty=1,label="gene",label.height=0.5,pad.for.label=T,cex=0.5) {
     xl <- xrange[1]
     xr <- xrange[2]
     label.pad <- 0
     if(pad.for.label) { label.pad <- label.height + transcript.pad}

     yb <-  offset
     yt <- (offset + (transcript.pad + transcript.height) * transcripts + label.pad + 2 * transcript.pad)
     if(strand == -1 ) {
       tmp <- -yb
       yb  <- -yt
       yt  <- tmp
     }
     rect(xleft=xl,ybot=yb,xright=xr,ytop=yt,col=fill,border=border,lty,density=-1)
     if(strand == 1 ) {
       text(xl,yt-transcript.pad-label.height,label,col=border,cex=cex,pos=4,offset=1)
     }
     else {
       text(xl,yb+transcript.pad+label.height,label,col=border,cex=cex,pos=4,offset=1)
     }
}


### ... and a transcript ...

.paint.transcript <- function(xrange,transcript=1,strand=1,offset=0.5,transcript.height=1,transcript.pad=0.1,border="black",fill="white",lty=1, draw.center.line=T,draw.box=T,center.lty=2,label="transcript",label.height=0.5,cex=cex) {
     xl <- xrange[1]
     xr <- xrange[2]
     if(strand == 1 ) {
       yb <- (offset + (transcript.pad + transcript.height) * (transcript - 1) + transcript.pad)
       yt <- (offset + (transcript.pad + transcript.height) * (transcript))
     }
     else { 
       yt <- -(offset + (transcript.pad + transcript.height) * (transcript - 1) + transcript.pad)
       yb <- -(offset + (transcript.pad + transcript.height) * (transcript))
     }
     if(draw.box) {
       rect(xleft=xl,ybot=yb,xright=xr,ytop=yt,col=fill,border=border,lty,density=-1)
     }

     if(draw.center.line) {
       if(strand == 1 ) {
         yc <- (yt-label.height-yb)/2 + yb
       }
       else {
         yc <- (yt+label.height-yb)/2 + yb
       }
       segments(xl,yc,xr,yc,lty=2,col=border)
     }
     if(strand == 1 ) {
       text(xl,yt-label.height,label,col=border,cex=cex,pos=4,offset=1)
     }
     else {
       text(xl,yb+label.height,label,col=border,cex=cex,pos=4,offset=1)
     }
}

### ...and an exon ...

.paint.exon <- function(xrange,transcript=1,utr3=NULL,utr5=NULL,no.data=F,strand=1,offset=0.5,transcript.height=1,transcript.pad=0.1,exon.pad=0.1,border="black",fill="white",no.data.colour="black",lty=1,space.for.transcript.label=T,transcript.label.height=0.5) {
     xl <- xrange[1]
     xr <- xrange[2]
     label.offset <- 0
     if(space.for.transcript.label) { label.offset <- transcript.label.height }
     if(strand == 1 ) {
       yb <- (offset + (transcript.pad + transcript.height) * (transcript - 1) + transcript.pad + exon.pad)
       yt <- (offset + (transcript.pad + transcript.height) * (transcript) - 2 * exon.pad - label.offset)
     }
     else {
       yt <- -(offset + (transcript.pad + transcript.height) * (transcript - 1) + transcript.pad + exon.pad)
       yb <- -(offset + (transcript.pad + transcript.height) * (transcript) - 2 * exon.pad - label.offset)
     }


     rect(xleft=xl,ybot=yb,xright=xr,ytop=yt,col=fill,border=border,lty,density=-1)

     if(class(utr3) == "numeric") {
        if(utr3[2] >= xl) {
          h <- abs(yt-yb)

          utrl <- xl
          utrr <- xr
          if(utr3[2] < utrr) { utrr <- utr3[2] }

          rect(xleft=utrl,ybot=yb + h/4,xright=utrr,ytop=yt - h/4,col=fill,border=border,lty=3,density=-1)
        }
     }

     if(class(utr5) == "numeric") {
        if(utr5[1] <= xr) {
          h <- abs(yt-yb)

          utrl <- xl
          utrr <- xr
          if(utr5[1] > utrl) { utrl <- utr5[1] }

          rect(xleft=utrl,ybot=yb + h/4,xright=utrr,ytop=yt - h/4,col=fill,border=border,lty=3,density=-1)
        }
     }
     if(no.data) {
        h <- abs(yt-yb)
        w <- (xr-xl)
        midy <- h/2 + yb
        rect(xleft=xl,ybot=yb,xright=xr,ytop=yt,col=fill,border=no.data.colour,lty=1,density=-1)

        segments(xl-w/4,midy+h,xr+w/4,midy-h,col=no.data.colour)
        segments(xl-w/4,midy-h,xr+w/4,midy+h,col=no.data.colour)
     }
}


### given an oject of expression data (x) and a probeset list, calculate mean (for example) fold change between two groups of arrays 
### (defined by gp1 and gp2 -- column indices into the expression data)

.pw.summary <- function(x,probesets,gp1,gp2,type=c("mean-fc","min-fc","max-fc","min-tt","mean-int","median-int","min-int","max-int")) {
  r <- c()
  probesets.in.eset(probesets,x)
  if(length(probesets) > 0) {
    type <- match.arg(type)
    r <- switch(type,
	  "mean-fc" = mean(apply(exprs(x)[as.character(probesets),,drop=F],1,function(d) { (mean(d[gp1])-mean(d[gp2])) })),

  	  "min-fc" = min(apply(exprs(x)[as.character(probesets),,drop=F],1,function(d) { min(d[gp1] - d[gp2]) })),

	  "max-fc" = max(apply(exprs(x)[as.character(probesets),,drop=F],1,function(d) { max(d[gp1] - d[gp2]) })),

	  "min-tt" = min(apply(exprs(x)[as.character(probesets),,drop=F],1,function(d) { t.test(d[gp1],d[gp2])$p.value })),

	  "mean-int" = mean(exprs(x)[as.character(probesets),gp1]),

	  "median-int" = median(exprs(x)[as.character(probesets),gp1]),

	  "min-int" = min(exprs(x)[as.character(probesets),gp1]),

	  "max-int" = max(exprs(x)[as.character(probesets),gp1]))
  }
  r
}

### map an expression level into a colour palette and get the appropriate colour out
col.rd.bl <- colorRampPalette(brewer.pal(10,"RdBu"))(101)
.value.to.colour <- function(x,cols=col.rd.bl,col.range=c(-5,5)) {
  idx <- x
  idx <- (idx - col.range[1]) /(col.range[2] - col.range[1])
  idx <- (length(cols) - 1) * idx
  idx <-floor(idx) + 1
  if(idx < 1) { idx <- 1 }
  if(idx > length(cols) ) { idx <- length(cols) }
  col <- cols[idx]
  col
}

# pull out the relevant bits from a gene.details table

.gd.summary <- function(geneid) {
  g <- gene.details(geneid)
  c(g[1,2],g[1,5],g[1,6],g[1,7],g[1,4],g[1,13])
}

### ... same for transcripts ...
.tr.summary <- function(transcriptid) {
  tr <- transcript.details(transcriptid)
  c(tr[1,2],tr[1,5],tr[1,6],tr[1,7])
}


### ...exons...
.ex.summary <- function(exonid) {
  ex <- exon.details(exonid)
  c(ex[1,2],ex[1,5],ex[1,6],ex[1,7])
}

### and probes
.pr.summary <- function(start,stop,strand,chr)  {
  probes.only.in.range(start, stop, strand, chr)[,c(1,3,4,5,6)]  
}


### remove any probesets from a probeset list that are not well behaved
### ie. multitarget, and/or don't hit an exon with all 4 probes

.filter.wbex <- function(probesets) {
  if(length(probesets) == 0 ) { return(c()) }
#  probesets <- exclude.probewise(probesets,"multitarget")
#  if(length(probesets) == 0 ) { return(c()) }
  probesets <- select.probewise(probesets,"exonic")
  if(length(probesets) == 0 ) { return(c()) }
  probesets
}

### Right, here goes
#   plot the gene specified, coloured using the expression levels in data and the groups defined by gp1 and gp2

plot.gene <- function(x,data,gp1,gp2,xlim,ylim,tick.labels=T,border="black",
                      show.probes=T,probe.colour="black",show.mt=T,mt.colour="grey",
                      type=c("mean-fc","min-fc","max-fc","min-tt","mean-int","median-int","min-int","max-int"),
                      use.only.wbex=T,palette=col.rd.bl,col.range,scale.to.gene=F,draw.exon.border=T,colour.transcript=F,
                      draw.legend=T,cex=0.5,...) {
  gene <- .gd.summary(x)
  transcripts <- gene.to.transcript(x)

  label <- paste(gene[[1]]," : ",gene[[6]],sep="")
  label.height <- par()$cin[2]

  gh <- .gene.height(transcripts=length(transcripts),strand=as.numeric(gene[[4]]),label.height=label.height,cex=cex)

  if(missing(xlim)) {
    xlim <- c(as.numeric(gene[[2]]),as.numeric(gene[[3]]))
  }  

  if(missing(ylim)) {

    if(as.numeric(gene[[4]]) == 1) { 
      if(draw.legend == T) { ylim <- c(-3,gh[2])   }
      else                 { ylim <- c(0,gh[2])    }
    }
    else{ 
      if(draw.legend == T) { ylim <- c(gh[1],3)   }
      else                 { ylim <- c(gh[1],0)   }
    } 
  }


  if(missing(col.range)) {
    ty <- match.arg(type)
    col.range <- switch(ty,
        "mean-fc"    = c(-5,5),
        "min-fc"     = c(-5,5),
        "max-fc"     = c(-5,5),
        "min-tt"     = c(0,0.1),
        "mean-int"   = c(0,14),
        "median-int" = c(0,14),
        "min-int"    = c(0,14),
        "max-int"    = c(0,14))
  }

  .initialize.plot.area(xlim=xlim,ylim=ylim,tick.labels=tick.labels,cex=cex)

  gene.val <- 0
  col <- "white"
  if(!missing(data)) {
    probesets <- probesets.in.eset(gene.to.probeset(x),data)
    if(use.only.wbex) {
      probesets <- .filter.wbex(probesets)
    }
    gene.val <- .pw.summary(data,probesets,gp1,gp2,type)
    if(scale.to.gene) { 
      col <- .value.to.colour(0,palette,col.range)
    }
    else  { 
      col <- .value.to.colour(gene.val,palette,col.range)
    }
  }

  probe.height <- max(abs(gh[1]),abs(gh[2])) + label.height
  all.probes <- .pr.summary(as.numeric(gene[[2]]),as.numeric(gene[[3]]),as.numeric(gene[[4]]),as.character(gene[[5]]))
  if(show.mt & show.probes) {
    apply(all.probes[all.probes[,5]!=1,],1,function(p) { .paint.probe(p[2:3],strand=p[4],colour=mt.colour,probe.height=probe.height*0.9)})
  }
  if(show.probes) {
    apply(all.probes[all.probes[,5]==1,],1,function(p) { .paint.probe(p[2:3],strand=p[4],colour=probe.colour,probe.height=probe.height)})
  }


  .paint.gene(c(as.numeric(gene[[2]]),as.numeric(gene[[3]])),transcripts=length(transcripts),strand=as.numeric(gene[[4]]),label=label,label.height=label.height,border=border,fill=col,cex=cex)

  pos <- 1
  for(i in transcripts) {  
    tr <- .tr.summary(i)

    col <- "white"
    if(!missing(data) & colour.transcript) {
      probesets <- probesets.in.eset(transcript.to.probeset(i),data)
      if(use.only.wbex) {
        probesets <- .filter.wbex(probesets)
      }
      if(length(probesets) > 0) {
        val <- .pw.summary(data,probesets,gp1,gp2,type)
        if(scale.to.gene) { 
          val <- val - gene.val
        }
        col <- .value.to.colour(val,palette,col.range)
      }
    }

    .paint.transcript(c(as.numeric(tr[[2]]),as.numeric(tr[[3]])),transcript=pos,strand=as.numeric(tr[[4]]),label=tr[[1]],label.height=label.height,fill=col,border=border,cex=cex)
     exons <- transcript.to.exon(tr)
     for(j in exons) {   
     ex <- .ex.summary(j)
     no.data <- F
     if(!missing(data)) {
       probesets <- probesets.in.eset(exon.to.probeset(j),data)
       if(use.only.wbex) {
         probesets <- .filter.wbex(probesets)
       }
       col <- "white"
       if(length(probesets) > 0) {
         val <- .pw.summary(data,probesets,gp1,gp2,type)
         if(scale.to.gene) { 
           val <- val - gene.val
         }
         no.data <- F 
         col <- .value.to.colour(val,palette,col.range)
       }
       else {
        no.data <- T
       }
     }
     if(draw.exon.border) { edge.colour <-  border }
     else { edge.colour <-  col }
     .paint.exon(c(as.numeric(ex[[2]]),as.numeric(ex[[3]])),transcript=pos,strand=as.numeric(ex[[4]]),no.data=no.data,fill=col,border=edge.colour,transcript.label.height=label.height)
    }
   pos <- pos + 1
  }
  if(draw.legend & !missing(data)) {
    le <- as.numeric(gene[[2]])
    ri <- as.numeric(gene[[3]])
    w <- (ri - le)/4
    gene.legend(x=le + (ri - le - w - w/5)/2,y=-2.5*as.numeric(gene[[4]]),w=w,h=0.5,palette=palette,col.range=col.range,label=type[1],cex=cex)
   }
}


# plot a colour bar

gene.legend <- function(x,y,w,h=1,palette,col.range,interval=100,cex=0.5,pad=h,label="colours") { 
  i    <- (col.range[2] - col.range[1])/interval
  idx  <- seq(col.range[1],col.range[2],i)
  i    <- w/interval
  x1    <- seq(x,x+w,i)
  x2    <- seq(x+i,x+w+i,i)

  cols <- sapply(idx,function(a) {.value.to.colour(a,palette,col.range)})
  mapply(function(pos1,pos2,col) {rect(pos1,y,pos2,y+h,col=col,border=col,density=-1) },x1,x2,cols)
  rect(x,y,x+w+i,y+h,density=-1,col=NULL,border="black")
  text(x,y,col.range[1],pos=1,cex=cex)
  text(x+w/8,y,col.range[1]   +   (col.range[2]-col.range[1])/8,pos=1,cex=cex)
  text(x+w/4,y,col.range[1]   +   (col.range[2]-col.range[1])/4,pos=1,cex=cex)
  text(x+3*w/8,y,col.range[1] + 3*(col.range[2]-col.range[1])/8,pos=1,cex=cex)
  text(x+w/2,y,col.range[1]   +   (col.range[2]-col.range[1])/2,pos=1,cex=cex)
  text(x+5*w/8,y,col.range[1] + 5*(col.range[2]-col.range[1])/8,pos=1,cex=cex)
  text(x+3*w/4,y,col.range[1] + 3*(col.range[2]-col.range[1])/4,pos=1,cex=cex)
  text(x+7*w/8,y,col.range[1] + 7*(col.range[2]-col.range[1])/8,pos=1,cex=cex)
  text(x+w,y,col.range[2],pos=1,cex=cex)
  text(x+w/2,y,label=label,pos=1,cex=cex,offset=1)
}

.initialize.gene.graph<-function(xlim,ylim,cex=0.5,cex.axis=0.5,xlab="",ylab="",...) {
  plot(0,type="n",xaxt="n",yaxt="n",bty="n",xlim=xlim,ylim=ylim,cex=cex,xlab=xlab,ylab=ylab,...)
  axis(2,cex.axis=cex.axis)
  axis(1,pos=0,cex.axis=cex.axis)
  segments(xlim[1],0,xlim[2],0)
}

.graph.paint.probeset <- function(probes,y,probeset.height=0.05,border="black",fill="white",cex=0.3,draw.probes=T,lty=3) {
    r <- range(probes)
    usr <- par()$usr
    probeset.height <- probeset.height * (usr[4] - usr[3]) / 2

    rect(r[1],y-probeset.height,r[2],y+probeset.height,border=border,col=fill,density=-1,lty=lty)
    if(draw.probes){
      points(probes,rep(y,length(probes)),pch=20,col=border,cex=cex)
    }
}

.graph.paint.exon <- function(xrange,exon.height=0.01,border="black",fill=NULL,lty=1,extend.vertical=T,vertical.colour="grey",vertical.lty=3) {
  usr <- par()$usr
  exon.height <- exon.height * (usr[4] - usr[3])
  rect(xrange[1],-exon.height,xrange[2],exon.height,lty=lty,density=-1,col=fill,border=border)
  if(extend.vertical) {
    segments(xrange[1],exon.height,xrange[1],usr[4],lty=vertical.lty,col=vertical.colour)
    segments(xrange[1],-exon.height,xrange[1],usr[3],lty=vertical.lty,col=vertical.colour)
    segments(xrange[2],exon.height,xrange[2],usr[4],lty=vertical.lty,col=vertical.colour)
    segments(xrange[2],-exon.height,xrange[2],usr[3],lty=vertical.lty,col=vertical.colour)
  }
}

plot.gene.graph <- function(x,data,gp1,gp2,xlim,ylim,
                      border="black",
                      show.mt=T,mt.colour="grey",
                      type=c("mean-fc","min-fc","max-fc","min-tt","mean-int","median-int","min-int","max-int"),
                      palette=col.rd.bl,col.range=c(-5,5),use.only.wb=T,scale.to.gene=F,
                      gp1.col="orange",
                      gp2.col="red",
                      draw.exon.border=T,exon.height=0.05,draw.legend=T,cex=0.5,xlab,ylab="",...) {
  gene <- .gd.summary(x)

  if(missing(xlim)) {
    xlim <- c(as.numeric(gene[[2]]),as.numeric(gene[[3]]))
  }  
  if(missing(ylim)) {
    ty <- match.arg(type)
    ylim = switch(ty,
        "mean-fc"    = c(-10,10),
        "min-fc"     = c(-10,10),
        "max-fc"     = c(-10,10),
        "min-tt"     = c(0,0.1),
        "mean-int"   = c(0,14),
        "median-int" = c(0,14),
        "min-int"    = c(0,14),
        "max-int"    = c(0,14))
  }

  gene.val <- 0
  col <- "white"
  probesets <- probesets.in.eset(gene.to.probeset(x),data)
  if(use.only.wb) {
    g.probesets <- exclude.probewise(probesets,"multitarget")
  }
  gene.val <- .pw.summary(data,g.probesets,gp1,gp2,type)
  if(missing(xlab)) {
    xlab <- paste(gene[[1]]," : ",gene[[6]],sep="")
  }
  .initialize.gene.graph(xlim,ylim,cex=cex,cex.axis=cex,xlab=xlab,ylab=ylab,...)

  exons <- gene.to.exon(x)
  for(j in exons) {   
    ex <- .ex.summary(j)

    probesets <- probesets.in.eset(exon.to.probeset(j),data)
    if(use.only.wb) {
      probesets <- exclude.probewise(probesets,"multitarget")
    }
    if(length(probesets) > 0) {
      val <- .pw.summary(data,probesets,gp1,gp2,type)
      if(scale.to.gene) { 
        val <- val - gene.val
      }
      col <- .value.to.colour(val,palette,col.range)
      if(draw.exon.border) { edge.colour <-  border }
      else { edge.colour <-  col }
      .graph.paint.exon(c(as.numeric(ex[[2]]),as.numeric(ex[[3]])),exon.height=0.1,border=edge.colour,fill=col)
    }
    else {
      .graph.paint.exon(c(as.numeric(ex[[2]]),as.numeric(ex[[3]])),exon.height=0.1,border=border,fill="white")
    }
  }
  ty <- match.arg(type)  
  if(ty == "median-int" | ty == "mean-int" | ty == "min-int" | ty == "max-int") {
    r <- sapply(g.probesets,function(ps) {
      probes <- probeset.to.probe(ps)
      y <- .pw.summary(data,ps,gp1,gp2,type)
      x <- probes[,6] + (probes[,7] - probes[,6]) /2
      .graph.paint.probeset(x,y,probeset.height=0,border="black",fill="white",cex=0.3,draw.probes=T,lty=3)
      cbind(x,y)
    },simplify=F)
    x <- sapply(r,function(a) { a[,1]})
    y <- sapply(r,function(a) { a[,2]})
    o <- order(x)
    x <- x[o]
    y <- y[o]

    polygon(c(x,rev(x)[1],x[1]),c(y,0,0),lty=3,density=0,col=gp1.col)
  
    r <- sapply(g.probesets,function(ps) {
      probes <- probeset.to.probe(ps)
      y <- .pw.summary(data,ps,gp2,gp1,type)
      x <- probes[,6] + (probes[,7] - probes[,6]) /2
      .graph.paint.probeset(x,y,probeset.height=0,border="black",fill="white",cex=0.3,draw.probes=T,lty=3)
      cbind(x,y)
    },simplify=F)

    x <- sapply(r,function(a) { a[,1]})
    y <- sapply(r,function(a) { a[,2]})
    o <- order(x)
    x <- x[o]
    y <- y[o]
    polygon(c(x,rev(x)[1],x[1]),c(y,0,0),lty=3,density=0,col=gp2.col)
  }
  else {

    r <- sapply(g.probesets,function(ps) {
      probes <- probeset.to.probe(ps)
      y <- .pw.summary(data,ps,gp1,gp2,type)
      x <- probes[,6] + (probes[,7] - probes[,6]) /2
      .graph.paint.probeset(x,y,probeset.height=0,border="black",fill="white",cex=0.3,draw.probes=T,lty=3)
      cbind(x,y)
    },simplify=F)

    x <- sapply(r,function(a) { a[,1]})
    y <- sapply(r,function(a) { a[,2]})
    o <- order(x)
    x <- x[o]
    y <- y[o]
    polygon(c(x,rev(x)[1],x[1]),c(y,0,0),lty=3,density=0,col=gp1.col)
  }
}

.initialize.gene.strip <- function(xlim,ylim,labels,cex=0.5,cex.axis=0.5,xlab="",ylab="",...) {
  if(missing(xlab)) { xlab="" }
  if(missing(ylab)) { ylab="" }
  plot(0,type="n",xaxs="i",yaxt="n",bty="l",xlim=xlim,ylim=ylim,cex=cex,cex.axis=cex.axis,xlab=xlab,ylab=ylab,...)
  axis(2,at=1:ylim[2],labels=labels,tick=FALSE,las=2,cex.axis=cex.axis)
}


.strip.paint.probeset <- function(probes,y,probeset.height=1,border="black",fill="white",outer.fill="white",probe.colour="black",cex=0.3,draw.probes=T,lty=3) {
    r <- range(probes)
    usr <- par()$usr
    rect(r[1],y-probeset.height/2,r[2],y+probeset.height/2,border=border,col=outer.fill,density=-1,lty=lty)
    rect(r[1],y-probeset.height/20,r[2],y+probeset.height/20,border=border,col=fill,density=-1,lty=lty)
    if(draw.probes){
      points(probes,rep(y,length(probes)),pch=20,col=probe.colour,cex=cex)
    }
}


.strip.paint.exon <- function(start,end,y,exon.height=0.5,border="black",fill=NULL,lty=1) {
  rect(start,y-exon.height/2,end,y+exon.height/2,col=fill,border=border,density=-1)
}

.plot.gene.strip.structure <- function(gene,y,start,end,border="black",lty=3) {
  lines(c(0,end-start),c(y,y),col=border,lty=lty)
  ex <- gene.to.exon(gene)
  for(i in ex) {
     ed <- .ex.summary(i)
     ex.start <- as.numeric(ed[2])
     ex.end   <- as.numeric(ed[3])
     if(end  > start) {
       .strip.paint.exon(ex.start-start,ex.end-start,y,border=border)
     }
  }
}


gene.strip <- function(X,data,gp1,gp2,plot.introns=FALSE,labels,type=c("mean-fc","min-fc","max-fc","min-tt","mean-int","median-int","min-int","max-int"),
                       scale.to.gene=FALSE,palette=col.rd.bl,col.range=c(-5,5),use.only.wbex=TRUE,exclude.mt=TRUE,
                       cex=0.5,xlab,ylab,border="black",plot=TRUE) {


  gds     <- t(sapply(X,function(a) { .gd.summary(a) }))
  lengths <- as.numeric(gds[,3]) - as.numeric(gds[,2])  
  longest <- max(abs(lengths))
  if(plot.introns) {
    xlim    <- c(0,longest)
    }
  else {
    no.exons <- sapply(X,function(a) { length(unique(gene.to.exon(a))) })
    max.exons <- max(no.exons)
    xlim    <- c(0,max.exons)
  }

  ylim    <- c(0.5,length(X)+0.5)
  if(missing(labels)) {
    strand <- sapply(as.numeric(gds[,4]), function(a) { if(a == 1) { ">" } else { "<" }})
    labels <-  paste(gds[,1],gds[,6],strand)
  }
  .initialize.gene.strip(xlim,ylim,labels=labels,cex=cex,cex.axis=cex,xlab,ylab)
  if(plot) {
    for(i in 1:length(X)) {
     if(plot.introns) {
       start <- as.numeric(gds[i,2])
       end  <-  as.numeric(gds[i,3])
     }
     else {
       start <- 0
       end   <- max.exons
     }

     gene.val <- 0
    
     col <- "white"
     if(plot.introns) {
       probesets <- probesets.in.eset(gene.to.probeset(X[i]),data)
       if(use.only.wbex) {
          probesets <- .filter.wbex(probesets)
       }
       else {
         if(exclude.mt) {
            probesets <- exclude.probewise(probesets,"multitarget")
         }
       }
       gene.val <- .pw.summary(data,probesets,gp1,gp2,type)  
       for(j in probesets) {
         probes <- probeset.to.probe(j)
         val <- .pw.summary(data,j,gp1,gp2,type)
         x   <- probes[,6] + (probes[,7] - probes[,6]) /2 - start
         y   <- i
         if(scale.to.gene) { 
           val <- val - gene.val
         }
         col <- .value.to.colour(val,palette,col.range)
         edge.colour <-  col
        .strip.paint.probeset(x,y,probeset.height=1,border=edge.colour,fill=edge.colour,lty=1)
       }

       .plot.gene.strip.structure(X[i],i,as.numeric(gds[i,2]),as.numeric(gds[i,3]),border=border)
     }
     else {
       exons <- exon.details(gene.to.exon(X[i]))
       exons <- unique(cbind(exons[,2],exons[,5]))
       exons <- exons[order(exons[,2],decreasing=FALSE),,drop=FALSE]

       for(j in 1:length(exons[,2])) {
         probesets <- probesets.in.eset(as.character(exon.to.probeset(exons[j,1])),data)
         if(use.only.wbex) {
            probesets <- .filter.wbex(probesets)
         }
         else {
           if(exclude.mt) {
              probesets <- exclude.probewise(probesets,"multitarget")
           }
         }
         gene.val <- .pw.summary(data,probesets,gp1,gp2,type)  
         height <- 1/length(probesets)
         y   <- i - 0.5 + height/2
         if(is.null(probesets)) {
           .strip.paint.exon(j-1,j,i,1,border,"white")
         }
         for(k in probesets) {
           val <- .pw.summary(data,k,gp1,gp2,type)
           x   <- j
           if(scale.to.gene) { 
             val <- val - gene.val
           }
           col <- .value.to.colour(val,palette,col.range)
           edge.colour <-  col

           .strip.paint.exon(x-1,x,y,height,border,edge.colour)
           y <- y + height
         }

       }
     }
    }
  }
}
